@@ -1,14 +1,14 @@
 import time
 import json
 
-from hashlib import md5
 from numbers import Number
 from collections import defaultdict
-from typing_extensions import Literal
 from typing import Tuple, Sequence, Any, Iterable, Dict, Union
+from coba.backport import Literal
 
-from coba.pipes import Pipe, Source, HttpIO, Default, Drop, Encode, _T_Data, Structure, ArffReader, CsvReader, Take
-from coba.config import CobaConfig
+
+from coba.pipes import Pipe, Source, HttpIO, Default, Drop, Encode, _T_Data, Structure, ArffReader, CsvReader, Reservoir
+from coba.contexts import CobaContext, CobaContext
 from coba.exceptions import CobaException
 from coba.encodings import NumericEncoder, OneHotEncoder, StringEncoder
 
@@ -104,7 +104,7 @@ class OpenmlSource(Source[Union[Iterable[Tuple[_T_Data, str]], Iterable[Tuple[_T
                 return "?" in row_values or "" in row_values
             
             drops     = Drop(drop_cols=ignored, drop_row=row_has_missing_values)
-            takes     = Take(self._take, keep_first=True, seed=1)
+            takes     = Reservoir(self._take, seed=1, keep_first=True)
             defaults  = Default({target:"0"})
             encodes   = Encode(encoders)
             structure = Structure([None, target])
@@ -123,7 +123,7 @@ class OpenmlSource(Source[Union[Iterable[Tuple[_T_Data, str]], Iterable[Tuple[_T
             #if something unexpected went wrong clear the cache just in case it was corrupted somehow
 
             for key in self._cache_keys.values():
-                CobaConfig.cacher.rmv(key)
+                CobaContext.cacher.rmv(key)
 
             raise
 
@@ -133,30 +133,31 @@ class OpenmlSource(Source[Union[Iterable[Tuple[_T_Data, str]], Iterable[Tuple[_T
         # For now we don't require cacher to be consistent so I'm commenting out for now. 
         # if checksum is not None and md5(bites).hexdigest() != checksum:
         #     #if the cache has become corrupted we need to clear it
-        #     CobaConfig.cacher.rmv(key)
+        #     CobaContext.cacher.rmv(key)
         #     message = (
         #         f"The response from {url} did not match the given checksum {checksum}. This could be the result "
         #         "of network errors or the file becoming corrupted. Please consider downloading the file again. "
         #         "If the error persists you may want to manually download and reference the file.")
         #     raise CobaException(message) from None
 
-        for b in CobaConfig.cacher.get_put(key, lambda: self._http_request(url)):
+        for b in CobaContext.cacher.get_put(key, lambda: self._http_request(url)):
             yield b.decode('utf-8')
 
     def _http_request(self, url:str) -> Iterable[bytes]:
-        api_key  = CobaConfig.api_keys['openml']
+        api_key = CobaContext.api_keys['openml']
+        srcsema = CobaContext.store.get("srcsema")
         
-        srcsema = CobaConfig.store.get("srcsema")
-        if srcsema: srcsema.acquire() # we only allow three paralellel request, another attempt at being more "considerate".
-            
+        # we only allow three paralellel request, another attempt at being more "considerate".
+        if srcsema:srcsema.acquire() 
+    
         # An attempt to be considerate of how often we hit their REST api. 
         # They don't publish any rate-limiting guidelines so this is just a guess.
-        time.sleep(1) 
+        if srcsema:time.sleep(1)
 
         try:
             with HttpIO(url + (f'?api_key={api_key}' if api_key else '')).read() as response:
 
-                if response.status_code == 412: # pragma: no cover
+                if response.status_code == 412:
                     if 'please provide api key' in response.text:
                         message = (
                             "Openml has requested an API Key to access openml's rest API. A key can be obtained by creating "
@@ -171,23 +172,24 @@ class OpenmlSource(Source[Union[Iterable[Tuple[_T_Data, str]], Iterable[Tuple[_T
                             "should be placed in ~/.coba as { \"api_keys\" : { \"openml\" : \"<your key here>\", } }.")
                         raise CobaException(message)
 
-                if response.status_code == 404: # pragma: no cover
-                    message = (
-                        "We're sorry but we were unable to find the requested dataset on openml.")
-                    raise CobaException(message)
+                if response.status_code == 404:
+                    raise CobaException("We're sorry but we were unable to find the requested dataset on openml.")
+
+                if response.status_code != 200:
+                    raise CobaException(f"An unexpected response was returned by openml: {response.text}")
 
                 # NOTE: These two checks need to be gated with a status code failure 
                 # NOTE: otherwise this will cause the data to be downloaded all at once
                 # NOTE: unfortunately I don't know the appropriate status code so commenting out for now
                 # if "Usually due to high server load" in response.text:
                 #     message = (
-                #         "Openml has experienced an error that they believe is the result of high server loads."
+                #         "Openml has experienced an error that they believe is the result of high server loads. "
                 #         "Openml recommends that you try again in a few seconds. Additionally, if not already "
-                #         "done, consider setting up a DiskCache in CobaConfig to reduce the number of openml "
-                #         "calls in the future.")
+                #         "done, consider setting up a DiskCache in a coba config file to reduce the number of "
+                #         "openml calls in the future.")
                 #     raise CobaException(message) from None
 
-                # if '' == response.text: # pragma: no cover
+                # if '' == response.text:
                 #     raise CobaException("Openml experienced an unexpected error. Please try requesting the data again.") from None
 
                 for b in response.iter_lines(decode_unicode=False):
@@ -219,10 +221,10 @@ class OpenmlSource(Source[Union[Iterable[Tuple[_T_Data, str]], Iterable[Tuple[_T
 
             openml_dialect = dict(quotechar="'", escapechar="\\", doublequote=False)
 
-            if arff_key in CobaConfig.cacher:
+            if arff_key in CobaContext.cacher:
                 return ArffReader(skip_encoding=True, **openml_dialect).filter(self._get_data(arff_url, arff_key, md5_checksum))
 
-            if csv_key in CobaConfig.cacher:
+            if csv_key in CobaContext.cacher:
                 return CsvReader(True, **openml_dialect).filter(self._get_data(csv_url, csv_key, md5_checksum))
 
             try:
@@ -245,7 +247,7 @@ class OpenmlSource(Source[Union[Iterable[Tuple[_T_Data, str]], Iterable[Tuple[_T
 
         raise CobaException(f"Openml {data_id} does not appear to be a {self._problem_type} dataset")
 
-    def __repr__(self) -> str:
+    def __str__(self) -> str:
         return f'{{"OpenmlSource":{self._data_id}}}'
 
 class OpenmlSimulation(SimulatedEnvironment):
@@ -276,8 +278,5 @@ class OpenmlSimulation(SimulatedEnvironment):
         else:
             return RegressionSimulation(self._source.read()).read()
 
-    def __repr__(self) -> str:
-        return f"OpenmlSimulation(id={self.params['openml']}, cat_as_str={self.params['cat_as_str']}, take={self.params.get('openml_take')})"
-
     def __str__(self) -> str:
-        return self.__repr__()
+        return f"OpenmlSimulation(id={self.params['openml']}, cat_as_str={self.params['cat_as_str']}, take={self.params.get('openml_take')})"
\ No newline at end of file
