@@ -1,9 +1,9 @@
 from typing import Iterable, Dict, Any
 
-from coba.pipes import Pipe, Identity
+from coba.pipes import Pipe
 
 from coba.environments.primitives import SimulatedEnvironment, SimulatedInteraction
-from coba.environments.filters import EnvironmentFilter
+from coba.environments.filters import EnvironmentFilter, Identity
 
 class EnvironmentPipe(SimulatedEnvironment):
 
@@ -45,9 +45,9 @@ class EnvironmentPipe(SimulatedEnvironment):
 
         return {}
 
-    def __repr__(self) -> str:
+    def __str__(self) -> str:
 
         str_source = str(self._source)
         str_filter = str(self._filter)
 
-        return ','.join(filter(None,[str_source,str_filter])).replace('{ Identity }','').replace(',,',',').strip(',')
\ No newline at end of file
+        return ','.join(filter(None,[str_source,str_filter])).replace('{ Identity }','').replace(',,',',').strip(',')
