@@ -1,12 +1,12 @@
 from pathlib import Path
-from typing_extensions import Literal
 from typing import Sequence, Optional
+from coba.backport import Literal
 
-from coba.config import CobaConfig
-from coba.pipes import Pipe
+from coba.pipes import Pipe, Foreach
 from coba.learners import Learner
 from coba.environments import Environment
-from coba.multiprocessing import CobaMultiprocessFilter
+from coba.multiprocessing import CobaMultiprocessor
+from coba.contexts import CobaContext, ExceptLog, StampLog, NameLog, DecoratedLogger
 
 from coba.experiments.process import CreateWorkItems,  RemoveFinished, ChunkByTask, ChunkBySource, ProcessWorkItems
 from coba.experiments.tasks   import EnvironmentTask, EvaluationTask, LearnerTask
@@ -66,15 +66,15 @@ class Experiment:
 
     @property
     def chunk_by(self) -> str:
-        return self._chunk_by if self._chunk_by is not None else CobaConfig.experiment.chunk_by
+        return self._chunk_by if self._chunk_by is not None else CobaContext.experiment.chunk_by
 
     @property
     def processes(self) -> int:
-        return self._processes if self._processes is not None else CobaConfig.experiment.processes
+        return self._processes if self._processes is not None else CobaContext.experiment.processes
 
     @property
     def maxtasksperchild(self) -> int:
-        return self._maxtasksperchild if self._maxtasksperchild is not None else CobaConfig.experiment.maxtasksperchild
+        return self._maxtasksperchild if self._maxtasksperchild is not None else CobaContext.experiment.maxtasksperchild
 
     def evaluate(self, result_file:str = None) -> Result:
         """Evaluate the experiment and return the results.
@@ -82,6 +82,12 @@ class Experiment:
         Args:
             result_file: The file we'd like to use for writing/restoring results experiments.
         """
+        cb, mp, mt = self.chunk_by, self.processes, self.maxtasksperchild
+
+        if mp > 1 or mt != 0:
+            CobaContext.logger = DecoratedLogger([ExceptLog()], CobaContext.logger, [NameLog(), StampLog()])
+        else:
+            CobaContext.logger = DecoratedLogger([ExceptLog()], CobaContext.logger, [StampLog()])
 
         restored = Result.from_file(result_file) if result_file and Path(result_file).exists() else Result()
 
@@ -92,25 +98,26 @@ class Experiment:
             assert n_given_learners     == restored.experiment['n_learners'    ], "The current experiment doesn't match the given transaction log."
             assert n_given_environments == restored.experiment['n_environments'], "The current experiment doesn't match the given transaction log."
 
-        cb, mp, mt = self.chunk_by, self.processes, self.maxtasksperchild
-
         workitems  = CreateWorkItems(self._environments, self._learners, self._learner_task, self._environment_task, self._evaluation_task)
         unfinished = RemoveFinished(restored)
         chunk      = ChunkByTask() if cb == 'task' else ChunkBySource()
         sink       = TransactionIO(result_file)
 
         single_process = ProcessWorkItems()
-        multi_process  = Pipe.join([chunk, CobaMultiprocessFilter(ProcessWorkItems(), mp, mt)])
+        multi_process  = Pipe.join([chunk, CobaMultiprocessor(ProcessWorkItems(), mp, mt)])
         process        = multi_process if mp > 1 or mt != 0 else single_process
 
         try:
-            if not restored: sink.write([["T0", n_given_learners, n_given_environments]])
-            Pipe.join(workitems, [unfinished, process], sink).run()
+            if not restored: sink.write(["T0", n_given_learners, n_given_environments])
+            Pipe.join(workitems, [unfinished, process], Foreach(sink)).run()
 
-        except KeyboardInterrupt: # pragma: no cover
-            CobaConfig.logger.log("Experiment execution was manually aborted via Ctrl-C")
+        except KeyboardInterrupt as e: # pragma: no cover
+            CobaContext.logger.log("Experiment execution was manually aborted via Ctrl-C")
         
         except Exception as ex: # pragma: no cover
-            CobaConfig.logger.log(ex)
+            CobaContext.logger.log(ex)
 
-        return sink.result
\ No newline at end of file
+        if isinstance(CobaContext.logger, DecoratedLogger):
+            CobaContext.logger = CobaContext.logger.undecorate()
+
+        return sink.read()
\ No newline at end of file
