@@ -6,8 +6,8 @@ from collections import defaultdict
 from typing import Iterable, Sequence, Any, Optional, Tuple, Union
 
 from coba.learners import Learner
-from coba.config import CobaConfig
-from coba.pipes import Source, Pipe, Filter
+from coba.contexts import CobaContext
+from coba.pipes import Source, Filter, SourceFilters
 from coba.environments import SimulatedEnvironment, EnvironmentPipe
 
 from coba.experiments.tasks import LearnerTask, EnvironmentTask, EvaluationTask
@@ -102,7 +102,7 @@ class ChunkBySource(Filter[Iterable[WorkItem], Iterable[Sequence[WorkItem]]]):
             yield list(sorted(chunk, key=lambda c: (c.environ_id, -1 if c.learner_id is None else c.learner_id)))
 
     def _get_source(self, env):
-        return env._source  if isinstance(env, (EnvironmentPipe, Pipe.SourceFilters)) else env
+        return env._source if isinstance(env, (EnvironmentPipe, SourceFilters)) else env
 
 class ChunkByTask(Filter[Iterable[WorkItem], Iterable[Iterable[WorkItem]]]):
 
@@ -138,7 +138,7 @@ class ProcessWorkItems(Filter[Iterable[WorkItem], Iterable[Any]]):
 
         if not chunk: return
 
-        with CobaConfig.logger.log(f"Processing chunk..."):
+        with CobaContext.logger.log(f"Processing chunk..."):
 
             for env_source, work_for_env_source in groupby(sorted(chunk, key=self._get_source_sort), key=self._get_source):
 
@@ -147,7 +147,7 @@ class ProcessWorkItems(Filter[Iterable[WorkItem], Iterable[Any]]):
                     if env_source is None:
                         loaded_source = None
                     else:
-                        with CobaConfig.logger.time(f"Loading {env_source}..."):
+                        with CobaContext.logger.time(f"Loading {env_source}..."):
                             #This is not ideal. I'm not sure how it should be improved so it is being left for now.
                             #Maybe add a flag to the Experiment to say whether the source should be stashed in mem?
                             loaded_source = list(env_source.read())
@@ -159,7 +159,7 @@ class ProcessWorkItems(Filter[Iterable[WorkItem], Iterable[Any]]):
                         if loaded_source is None:
                             interactions = []
                         else:
-                            with CobaConfig.logger.time(f"Creating Environment {env_id} from Loaded Source..."):
+                            with CobaContext.logger.time(f"Creating Environment {env_id} from Loaded Source..."):
                                 interactions = list(env_filter.filter(loaded_source)) if env_filter else loaded_source
 
                             if len(filter_groups) == 1:
@@ -168,37 +168,37 @@ class ProcessWorkItems(Filter[Iterable[WorkItem], Iterable[Any]]):
                                 gc.collect()
 
                             if not interactions:
-                                CobaConfig.logger.log(f"Environment {env_id} has nothing to evaluate (this is often due to `take` being larger than source).")
-                                return
+                                CobaContext.logger.log(f"Environment {env_id} has nothing to evaluate (this is often due to `take` being larger than source).")
+                                break
 
                         for workitem in work_for_env_filter:
                             try:
 
                                 if workitem.environ is None:
-                                    with CobaConfig.logger.time(f"Recording Learner {workitem.learner_id} parameters..."):
+                                    with CobaContext.logger.time(f"Recording Learner {workitem.learner_id} parameters..."):
                                         row = workitem.task.process(deepcopy(workitem.learner))
                                         yield ["T1", workitem.learner_id, row]
 
                                 if workitem.learner is None:
-                                    with CobaConfig.logger.time(f"Recording Environment {workitem.environ_id} statistics..."):
+                                    with CobaContext.logger.time(f"Recording Environment {workitem.environ_id} statistics..."):
                                         row = workitem.task.process(workitem.environ,interactions)
                                         yield ["T2", workitem.environ_id, row]
 
                                 if workitem.environ and workitem.learner:
-                                    with CobaConfig.logger.time(f"Evaluating Learner {workitem.learner_id} on Environment {workitem.environ_id}..."):
+                                    with CobaContext.logger.time(f"Evaluating Learner {workitem.learner_id} on Environment {workitem.environ_id}..."):
                                         row = list(workitem.task.process(deepcopy(workitem.learner), interactions))
                                         yield ["T3", (workitem.environ_id, workitem.learner_id), row]
 
                             except Exception as e:
-                                CobaConfig.logger.log(e)
+                                CobaContext.logger.log(e)
 
                 except Exception as e:
-                    CobaConfig.logger.log(e)
+                    CobaContext.logger.log(e)
 
     def _get_source(self, task:WorkItem) -> SimulatedEnvironment:
         if task.environ is None:
             return None
-        elif isinstance(task.environ, (Pipe.SourceFilters, EnvironmentPipe)):
+        elif isinstance(task.environ, (SourceFilters, EnvironmentPipe)):
             return task.environ._source 
         else:
             return task.environ
@@ -209,7 +209,7 @@ class ProcessWorkItems(Filter[Iterable[WorkItem], Iterable[Any]]):
     def _get_id_filter(self, task:WorkItem) -> Tuple[int, Filter[SimulatedEnvironment,SimulatedEnvironment]]:
         if task.environ is None:
             return (-1,None)
-        elif isinstance(task.environ, (Pipe.SourceFilters, EnvironmentPipe)):
+        elif isinstance(task.environ, (SourceFilters, EnvironmentPipe)):
             return (task.environ_id, task.environ._filter) 
         else:
             return (task.environ_id, None)
