@@ -1,59 +1,48 @@
-from logging import raiseExceptions
 import re
 import collections
+import collections.abc
 
-from pathlib import Path
 from copy import copy
+from pathlib import Path
 from numbers import Number
 from operator import truediv
 from itertools import chain, repeat, accumulate
-from collections.abc import Container
-from typing_extensions import Literal
-from typing import Any, Iterable, Dict, List, Tuple, Optional, Sequence, Hashable, Iterator, Union, Type, Callable
+from typing import Any, Dict, List, Tuple, Optional, Sequence, Hashable, Iterator, Union, Type, Set, Callable
+from coba.backport import Literal
 
-from coba.config import CobaConfig
+from coba.contexts import CobaContext
 from coba.exceptions import CobaException
 from coba.utilities import PackageChecker
-from coba.pipes import JsonEncode, JsonDecode, DiskIO, MemoryIO, Sink, Source
+from coba.pipes import JsonEncode, JsonDecode, DiskIO, MemoryIO, IO
 
 class Table:
     """A container class for storing tabular data."""
 
-    def __init__(self, name:str, primary_cols: Sequence[str], rows: Sequence[Dict[str,Any]] = [], prefered_cols: Sequence[str] = []):
+    def __init__(self, name:str, primary_cols: Sequence[str], rows: Sequence[Dict[str,Any]], preferred_cols: Sequence[str] = []):
         """Instantiate a Table.
-        
+
         Args:
-            name: The name of the table.
-            default: The default values to fill in missing values with
+            name: The name of the table. Used for display purposes.
+            primary_cols: Table columns used to make each row's tuple "key".
+            rows: The actual rows that should be stored in the table. Each row is required to contain the given primary_cols.
+            preferred_cols: A list of columns that we prefer be displayed immediately after primary columns. All remaining 
+                columns (i.e., neither primary nor preferred) will be ordered alphabetically.
         """
         self._name    = name
         self._primary = primary_cols
 
-        def index_cols():
-            for row in rows:
-                if '_packed' in row: 
-                    return ['index']
-            return []
-        
-        def data_cols():
-            return ( sorted(row.keys() - ['_packed'] | row.get('_packed',{}).keys()) for row in rows)
-
         for row in rows:
             assert len(row.keys() & primary_cols) == len(primary_cols), 'A Table row was provided without a primary key.'
 
-        all_columns   = list(chain(primary_cols, index_cols(), *data_cols()))
-        
-        def col_order(col):
-            if col in primary_cols:
-                return primary_cols.index(col)
-            
-            if col in prefered_cols:
-                return len(primary_cols) + prefered_cols.index(col)
+        all_columns: Set[str] = set()
+        for row in rows:
+            all_columns |= {'index'} if '_packed' in row else set()
+            all_columns |= row.keys()-{'_packed'}
+            all_columns |= all_columns.union(row.get('_packed',{}).keys())
 
-            return len(primary_cols) + len(prefered_cols) + all_columns.index(col)
-
-        self._columns = sorted(set(all_columns), key=col_order)
+        col_priority = list(chain(primary_cols + ['index'] + preferred_cols + sorted(all_columns)))
 
+        self._columns = sorted(all_columns, key=col_priority.index)
         self._rows_keys: List[Hashable               ] = []               
         self._rows_flat: Dict[Hashable, Dict[str,Any]] = {}
         self._rows_pack: Dict[Hashable, Dict[str,Any]] = {}
@@ -119,11 +108,11 @@ class Table:
             col_filter_results = [ ]
 
             for col,col_filter in kwargs.items():
-                
-                if isinstance(col_filter,Container) and not isinstance(col_filter,str):
-                    
+
+                if isinstance(col_filter,collections.abc.Container) and not isinstance(col_filter,str):
+
                     col_filter_results.append(row[col] in col_filter or any([satisifies_filter(cf,row[col]) for cf in col_filter]))
-                
+
                 else:
                     col_filter_results.append(satisifies_filter(col_filter,row.get(col,self._default(col)) ))
 
@@ -226,6 +215,10 @@ class Table:
     def __str__(self) -> str:
         return str({"Table": self.name, "Columns": self.columns, "Rows": len(self)})
 
+    def _ipython_display_(self):
+        #pretty print in jupyter notebook (https://ipython.readthedocs.io/en/stable/config/integrating.html)
+        print(str(self))
+
     def __len__(self) -> int:
         return sum([ len(self._rows_pack[key].get('index',[None])) for key in self.keys ])
 
@@ -242,13 +235,13 @@ class InteractionsTable(Table):
         lrn_sim_rows = []
 
         for interactions in self:
-            
+
             rewards = interactions[yaxis]
 
             if span is None or span >= len(rewards):
                 cumwindow  = list(accumulate(rewards))
                 cumdivisor = list(range(1,len(cumwindow)+1))
-            
+
             elif span == 1:
                 cumwindow  = list(rewards)
                 cumdivisor = [1]*len(cumwindow)
@@ -262,10 +255,10 @@ class InteractionsTable(Table):
 
         if each:
             return lrn_sim_rows
-        
+
         else:
             grouped_lrn_sim_rows = collections.defaultdict(list)
-            
+
             for row in lrn_sim_rows:
                 grouped_lrn_sim_rows[row[0]].append(row[2:])
 
@@ -274,7 +267,7 @@ class InteractionsTable(Table):
             for learner_id in grouped_lrn_sim_rows.keys():
 
                 Z = list(zip(*grouped_lrn_sim_rows[learner_id]))
-                
+
                 if not Z: continue
 
                 Y = [ sum(z)/len(z) for z in Z ]
@@ -298,35 +291,30 @@ class InteractionsTable(Table):
             n_index = len(data[0][1:])
             return pd.DataFrame(data, columns=["learner_id", *range(1,n_index+1)])
 
-class TransactionIO_V3(Source[Iterable[Any]],Sink[Iterable[Any]]):
+class TransactionIO_V3(IO['Result', Any]):
 
     def __init__(self, transaction_log: Optional[str] = None, minify:bool=True) -> None:
 
         self._io = DiskIO(transaction_log) if transaction_log else MemoryIO()
         self._minify = minify
 
-    def write(self, items: Iterable[Any]) -> None:
+    def write(self, item: Any) -> None:
         if isinstance(self._io, MemoryIO):
-            self._io.write(map(self._encode, items))
-        else:
-            self._io.write(map(JsonEncode(self._minify).filter, map(self._encode,items)))
+            self._io.write(self._encode(item))
+        else: 
+            if not Path(self._io._filename).exists():self._io.write('["version",3]')
+            self._io.write(JsonEncode(self._minify).filter(self._encode(item)))
 
-    def read(self) -> Iterable[Any]:
-        if isinstance(self._io, MemoryIO):
-            return self._io.read()
-        else:
-            return map(JsonDecode().filter,self._io.read())
-
-    @property
-    def result(self) -> 'Result':
-        
+    def read(self) -> 'Result':
         n_lrns   = None
         n_sims   = None
         lrn_rows = {}
         sim_rows = {}
         int_rows = {}
 
-        for trx in self.read():
+        for trx in self._io.read() if isinstance(self._io, MemoryIO) else map(JsonDecode().filter,self._io.read()):
+
+            if not trx: continue
 
             if trx[0] == "benchmark": 
                 n_lrns = trx[1]["n_learners"]
@@ -346,7 +334,7 @@ class TransactionIO_V3(Source[Iterable[Any]],Sink[Iterable[Any]]):
     def _encode(self,item):
         if item[0] == "T0":
             return ['benchmark', {"n_learners":item[1], "n_simulations":item[2]}]
-        
+
         if item[0] == "T1":
             return ["L", item[1], item[2]]
 
@@ -364,34 +352,32 @@ class TransactionIO_V3(Source[Iterable[Any]],Sink[Iterable[Any]]):
 
             return ["I", item[1], { "_packed": rows_T }]
 
-class TransactionIO_V4(Source[Iterable[Any]],Sink[Iterable[Any]]):
+        return None
+
+class TransactionIO_V4(IO['Result', Any]):
 
     def __init__(self, transaction_log: Optional[str] = None, minify:bool=True) -> None:
-        self._io = DiskIO(transaction_log) if transaction_log else MemoryIO()
+        self._io     = DiskIO(transaction_log) if transaction_log else MemoryIO()
         self._minify = minify
 
-    def write(self, items: Iterable[Any]) -> None:
+    def write(self, item: Any) -> None:
         if isinstance(self._io, MemoryIO):
-            self._io.write(map(self._encode, items))
-        else:
-            self._io.write(map(JsonEncode(self._minify).filter, map(self._encode,items)))
+            self._io.write(self._encode(item))
+        else: 
+            if not Path(self._io._filename).exists():self._io.write('["version",4]')
+            self._io.write(JsonEncode(self._minify).filter(self._encode(item)))
 
-    def read(self) -> Iterable[Any]:
-        if isinstance(self._io, MemoryIO):
-            return self._io.read()
-        else:
-            return map(JsonDecode().filter,self._io.read())
+    def read(self) -> 'Result':
 
-    @property
-    def result(self) -> 'Result':
-        
         n_lrns   = None
         n_sims   = None
         lrn_rows = {}
         env_rows = {}
         int_rows = {}
 
-        for trx in self.read():
+        for trx in self._io.read() if isinstance(self._io, MemoryIO) else map(JsonDecode().filter,self._io.read()):
+
+            if not trx: continue
 
             if trx[0] == "experiment": 
                 n_lrns = trx[1]["n_learners"]
@@ -411,7 +397,7 @@ class TransactionIO_V4(Source[Iterable[Any]],Sink[Iterable[Any]]):
     def _encode(self,item):
         if item[0] == "T0":
             return ['experiment', {"n_learners":item[1], "n_environments":item[2]}]
-        
+
         if item[0] == "T1":
             return ["L", item[1], item[2]]
 
@@ -429,9 +415,9 @@ class TransactionIO_V4(Source[Iterable[Any]],Sink[Iterable[Any]]):
 
             return ["I", item[1], { "_packed": rows_T }]
 
-        return item
+        return None
 
-class TransactionIO(Source[Iterable[Any]],Sink[Iterable[Any]]):
+class TransactionIO(IO['Result', Any]):
 
     def __init__(self, transaction_log: Optional[str] = None) -> None:
 
@@ -448,41 +434,15 @@ class TransactionIO(Source[Iterable[Any]],Sink[Iterable[Any]]):
 
         elif version is None:
             self._transactionIO = TransactionIO_V4(transaction_log)
-            self._transactionIO.write([['version',4]])
 
         else:
             raise CobaException("We were unable to determine the appropriate Transaction reader for the file.")
 
-    def write(self, items: Iterable[Any]) -> None:
-        self._transactionIO.write(items)
+    def write(self, transaction: Any) -> None:
+        self._transactionIO.write(transaction)
 
-    def read(self) -> Iterable[Any]:
-        self._transactionIO.read()
-
-    @property
-    def result(self) -> 'Result':
-        return self._transactionIO.result
-
-    def _encode(self,item):
-        if item[0] == "T0":
-            return ['benchmark', {"n_learners":item[1], "n_simulations":item[2]}]
-        
-        if item[0] == "T1":
-            return ["L", item[1], item[2]]
-
-        if item[0] == "T2":
-            return ["S", item[1], item[2]]
-
-        if item[0] == "T3":
-            rows_T = collections.defaultdict(list)
-
-            for row in item[2]:
-                for col,val in row.items():
-                    if col == "rewards" : col="reward"
-                    if col == "reveals" : col="reveal"
-                    rows_T[col].append(val)
-
-            return ["I", item[1], { "_packed": rows_T }]
+    def read(self) -> 'Result':
+        return self._transactionIO.read()
 
 class Result:
     """A class representing the result of an Experiment."""
@@ -490,11 +450,11 @@ class Result:
     @staticmethod
     def from_file(filename: str) -> 'Result':
         """Create a Result from a transaction file."""
-        
+
         if not Path(filename).exists(): 
             raise CobaException("We were unable to find the given Result file.")
 
-        return TransactionIO(filename).result
+        return TransactionIO(filename).read()
 
     def __init__(self,
         n_lrns  : int = None,
@@ -539,12 +499,12 @@ class Result:
         """
         return self._interactions
 
-    def _copy(self) -> 'Result':
+    def copy(self) -> 'Result':
         result = Result()
 
-        result.environments = copy(self._environments)
-        result.learners     = copy(self._learners)
-        result.interactions = copy(self._interactions)
+        result._environments = copy(self._environments)
+        result._learners     = copy(self._learners)
+        result._interactions = copy(self._interactions)
 
         return result
 
@@ -558,7 +518,7 @@ class Result:
         new_result._interactions = self.interactions.filter(environment_id=is_complete_sim)
 
         if len(new_result.environments) == 0:
-            CobaConfig.logger.log(f"No simulation was found with interaction data for every learner.")
+            CobaContext.logger.log(f"There was no environment which was finished for every learner.")
 
         return new_result
 
@@ -569,7 +529,7 @@ class Result:
         new_result._interactions = new_result.interactions.filter(environment_id=new_result.environments)
 
         if len(new_result.environments) == 0:
-            CobaConfig.logger.log(f"No environments matched the given filter: {kwargs}.")
+            CobaContext.logger.log(f"No environments matched the given filter.")
 
         return new_result
 
@@ -579,7 +539,7 @@ class Result:
         new_result._interactions = new_result.interactions.filter(learner_id=new_result.learners)
 
         if len(new_result.learners) == 0:
-            CobaConfig.logger.log(f"No learners matched the given filter: {kwargs}.")
+            CobaContext.logger.log(f"No learners matched the given filter.")
 
         return new_result
 
@@ -601,105 +561,112 @@ class Result:
             span: In general this indicates how many previous evaluations to average together. In practice this works
                 identically to ewm span value in the Pandas API. Additionally, if span equals None then all previous 
                 rewards are averaged together vs span = 1 WHERE the instantaneous reward is plotted for each interaction.
-            err: Determine what kind of error bars to plot (if any). Valid types are `None`, 'se', and 'sd'. If `None`
+            err: This determines what kind of error bars to plot (if any). Valid types are `None`, 'se', and 'sd'. If `None`
                 then no bars are plotted, if 'se' the standard error is shown, and if 'sd' the standard deviation is shown.
-            each: Determine whether each constituent observation used to estimate mean performance is also plotted.
+            each: This determines whether each evaluated environment used to estimate mean performance is also plotted.
             filename: Provide a filename to write plot image to disk.
             ax: Provide an optional axes that the plot will be drawn to. If not provided a new figure/axes is created.
         """
 
         PackageChecker.matplotlib('Result.plot_learners')
         import matplotlib.pyplot as plt #type: ignore
-        import numpy as np              #type: ignore
 
-        progressives: Dict[int,List[Sequence[float]]] = collections.defaultdict(list)
-
-        for progressive in self.interactions.to_progressive_lists(span=span,each=True):
-            progressives[progressive[0]].append(progressive[2:])
-
-        if not progressives:
-            return
-        
         show = ax is None
 
-        if ax is None:
-            ax  = plt.figure(figsize=(10,6)).add_subplot(111) #type: ignore
-
-        for learner_id in sorted(self.learners.keys, key=lambda id: self.learners[id]["full_name"]):
+        for label, X, Y, yerr, Z in self._plot_learners_data(xlim,span,err):
 
+            ax = ax or plt.figure(figsize=(10,6)).add_subplot(111) #type: ignore
+            
             color = next(ax._get_lines.prop_cycler)['color']
 
-            label = self._learners[learner_id]["full_name"]
-            Z     = list(zip(*progressives[learner_id]))
-            
-            if not Z: continue
-
-            N     = [ len(z) for z in Z        ]
-            Y     = [ sum(z)/len(z) for z in Z ]
-            X     = list(range(1,len(Y)+1))
-
-            start = xlim[0] if xlim else int(.05*len(X))
-            end   = xlim[1] if xlim else len(X)
-
-            if start >= end:
-                CobaConfig.logger.log("The plot's end is less than the start making plotting impossible.")
-                return
-
-            X = X[start:end]
-            Y = Y[start:end]
-            Z = Z[start:end]
-
-            if len(X) == 0: continue
-
-            #this is much faster than python's native stdev
-            #and more or less free computationally so we always
-            #calculate it regardless of if they are showing them
-            #we are using the identity Var[Y] = E[Y^2]-E[Y]^2
-            Y2 = [ sum([zz**2 for zz in z])/len(z) for z in Z            ]
-            SD = [ (round(y2-y**2,8))**(1/2)       for y,y2 in zip(Y,Y2) ]
-            SE = [ sd/(n**(1/2))                   for sd,n in zip(SD,N) ]
-
-            yerr = 0 if err is None else SE if err.lower() == 'se' else SD if err.lower() == 'sd' else 0
             ax.errorbar(X, Y, yerr=yerr, elinewidth=0.5, errorevery=(0,max(int(len(X)*0.05),1)), label=label, color=color)
 
             if each:
                 for Y in list(zip(*Z)):
                     ax.plot(X,Y, color=color, alpha=0.15)
 
-        padding = .05
-        ax.margins(0)
-        ax.set_xticks(np.clip(ax.get_xticks(), *ax.get_xlim()))
-        ax.margins(padding)
-
-        if xlim:
-            x_pad = padding*(xlim[1]-xlim[0])
-            ax.set_xlim(xlim[0]-x_pad, xlim[1]+x_pad)
-
-        if ylim:
-            y_pad = padding*(ylim[1]-ylim[0])
-            ax.set_ylim(ylim[0]-y_pad, ylim[1]+y_pad)
-
-        ax.set_title(("Instantaneous" if span == 1 else "Progressive" if span is None else f"Span {span}") + " Reward", loc='left',pad=15)
-        ax.set_ylabel("Reward")
-        ax.set_xlabel("Interactions")
-
-        if ax.get_legend() is None:
-            scale = 0.65
-            box1 = ax.get_position()
-            ax.set_position([box1.x0, box1.y0 + box1.height * (1-scale), box1.width, box1.height * scale])
+        if ax is None:
+            CobaContext.logger.log(f"No data was found for plotting in the given results: {self}.")
         else:
-            ax.get_legend().remove()
+            padding = .05
+            ax.margins(0)
+            ax.set_xticks([min(ax.get_xlim()[1], max(ax.get_xlim()[0],x)) for x in ax.get_xticks()])
+            ax.margins(padding)
 
-        ax.legend(*ax.get_legend_handles_labels(), loc='upper left', bbox_to_anchor=(-.01, -.25), ncol=1, fontsize='medium') #type: ignore
+            if xlim:
+                x_pad = padding*(xlim[1]-xlim[0])
+                ax.set_xlim(xlim[0]-x_pad, xlim[1]+x_pad)
 
-        if show:
-            plt.show()
+            if ylim:
+                y_pad = padding*(ylim[1]-ylim[0])
+                ax.set_ylim(ylim[0]-y_pad, ylim[1]+y_pad)
 
-        if filename:
-            plt.savefig(filename, dpi=300)
+            ax.set_title(("Instantaneous" if span == 1 else "Progressive" if span is None else f"Span {span}") + " Reward", loc='left',pad=15)
+            ax.set_ylabel("Reward")
+            ax.set_xlabel("Interactions")
+
+            if ax.get_legend() is None:
+                scale = 0.65
+                box1 = ax.get_position()
+                ax.set_position([box1.x0, box1.y0 + box1.height * (1-scale), box1.width, box1.height * scale])
+            else:
+                ax.get_legend().remove()
+
+            ax.legend(*ax.get_legend_handles_labels(), loc='upper left', bbox_to_anchor=(-.01, -.25), ncol=1, fontsize='medium') #type: ignore
+
+            if show:
+                plt.show()
+
+            if filename:
+                plt.savefig(filename, dpi=300)
+
+    def _plot_learners_data(self, xlim: Tuple[Number,Number] = None, span: int = None, err: Literal['se','sd'] = None):
+
+        if xlim and xlim[0] >= xlim[1]:
+            CobaContext.logger.log("The given x-limit end is less than the x-limit start. Plotting is impossible.")
+
+        progressives: Dict[int,List[Sequence[float]]] = collections.defaultdict(list)
+
+        for progressive in self.interactions.to_progressive_lists(span=span,each=True):
+            progressives[progressive[0]].append(progressive[2:])
+
+        if progressives and (not xlim or xlim[0] < xlim[1]):
+
+            for learner_id in sorted(self.learners.keys, key=lambda id: self.learners[id]["full_name"]):
+
+                label = self._learners[learner_id]["full_name"]
+                Z     = list(zip(*progressives[learner_id]))
+                
+                if not Z: continue
+
+                N     = [ len(z) for z in Z        ]
+                Y     = [ sum(z)/len(z) for z in Z ]
+                X     = list(range(1,len(Y)+1))
+
+                start = xlim[0] if xlim else int(.05*len(X))
+                end   = xlim[1] if xlim else len(X)
+
+                X = X[start:end]
+                Y = Y[start:end]
+                Z = Z[start:end]
+
+                if len(X) == 0: continue
+
+                #this is much faster than python's native stdev
+                #and more or less free computationally so we always
+                #calculate it regardless of if they are showing them
+                #we are using the identity Var[Y] = E[Y^2]-E[Y]^2
+                Y2 = [ sum([zz**2 for zz in z])/len(z) for z in Z            ]
+                SD = [ (round(y2-y**2,8))**(1/2)       for y2,y in zip(Y2,Y) ]
+                SE = [ sd/(n**(1/2))                   for sd,n in zip(SD,N) ]
+
+                yerr = 0 if err is None else SE if err.lower() == 'se' else SD if err.lower() == 'sd' else 0
+
+                yield label, X, Y, yerr, Z  
 
     def __str__(self) -> str:
         return str({ "Learners": len(self._learners), "Environments": len(self._environments), "Interactions": len(self._interactions) })
 
-    def __repr__(self) -> str:
-        return str(self)
+    def _ipython_display_(self):
+        #pretty print in jupyter notebook (https://ipython.readthedocs.io/en/stable/config/integrating.html)
+        print(str(self))
