@@ -6,8 +6,7 @@ inherit from this interface to implement new learners. In addition, a number
 of learners are provided out of the box for testing and baseline comparisons.
 """
 
-from coba.learners.primitives import Learner
-from coba.learners.core import SafeLearner
+from coba.learners.primitives import Learner, SafeLearner
 from coba.learners.bandit import EpsilonBanditLearner, UcbBanditLearner, FixedLearner, RandomLearner
 from coba.learners.corral import CorralLearner
 from coba.learners.vowpal import VowpalLearner, VowpalMediator
