@@ -3,10 +3,14 @@
 import math
 
 from typing import Any, Sequence, Optional, Dict, Tuple
+from coba.backport import Literal
 
+from coba.exceptions import CobaException
 from coba.random import CobaRandom
 from coba.environments import Context, Action
-from coba.learners.core import Learner, SafeLearner, Probs, Info
+from coba.contexts import LearnerContext
+
+from coba.learners.primitives import Learner, SafeLearner, Probs, Info
 
 class CorralLearner(Learner):
     """This is an implementation of the Agarwal et al. (2017) Corral algorithm.
@@ -20,7 +24,12 @@ class CorralLearner(Learner):
         Theory, pp. 12-38. PMLR, 2017.
     """
 
-    def __init__(self, base_learners: Sequence[Learner], eta: float, T: float = math.inf, type:str="importance", seed: int = 1) -> None:
+    def __init__(self, 
+        base_learners: Sequence[Learner], 
+        eta          : float = 0.075,
+        T            : float = math.inf, 
+        type         : Literal["importance","rejection","off-policy"] ="importance", 
+        seed         : int = 1) -> None:
         """Instantiate a CorralLearner.
 
         Args:
@@ -34,8 +43,8 @@ class CorralLearner(Learner):
                 original paper used importance sampling. We also support `off-policy` and `rejection`.
             seed: A seed for a random number generation in ordre to get repeatable results.
         """
-
-        assert type in ["importance", "off-policy", "rejection"], "The provided `type` for CorralLearner was unrecognized."
+        if type not in ["importance", "off-policy", "rejection"]:
+            raise CobaException("The provided `type` for CorralLearner was unrecognized.")
 
         self._base_learners = [ SafeLearner(learner) for learner in base_learners]
 
@@ -62,7 +71,7 @@ class CorralLearner(Learner):
 
         See the base class for more information
         """
-        return { "family": "corral", "eta": self._eta_init, "type":self._type, "T": self._T, "B": [ b.params["family"] for b in self._base_learners ], "seed":self._random_pick._seed }
+        return { "family": "corral", "eta": self._eta_init, "type":self._type, "T": self._T, "B": [ str(b) for b in self._base_learners ], "seed":self._random_pick._seed }
 
     def predict(self, context: Context, actions: Sequence[Action]) -> Tuple[Probs, Info]:
         """Determine a PMF with which to select the given actions.
@@ -78,16 +87,16 @@ class CorralLearner(Learner):
         base_predicts = [ base_algorithm.predict(context, actions) for base_algorithm in self._base_learners ]
         base_predicts, base_infos = zip(*base_predicts)
 
-        base_actions = [ self._random_pick.choice(actions, predict) for predict in base_predicts              ]
-        base_probs   = [ predict[actions.index(action)] for action,predict in zip(base_actions,base_predicts) ]
-
         if self._type in ["importance"]:
+            base_actions = [ self._random_pick.choice(actions, predict) for predict in base_predicts              ]
+            base_probs   = [ predict[actions.index(action)] for action,predict in zip(base_actions,base_predicts) ]
+
             predict = [ sum([p_b*int(a==b_a) for p_b,b_a in zip(self._p_bars, base_actions)]) for a in actions ]
             info    = (base_actions, base_probs, base_infos, base_predicts, actions, predict)
 
         if self._type in ["off-policy", "rejection"]:
             predict = [ sum([p_b*b_p[i] for p_b,b_p in zip(self._p_bars, base_predicts)]) for i in range(len(actions)) ]
-            info    = (base_actions, base_probs, base_infos, base_predicts, actions, predict)
+            info    = (None, None, base_infos, base_predicts, actions, predict)
 
         return (predict, info)
 
@@ -111,8 +120,6 @@ class CorralLearner(Learner):
         actions      = info[4]
         predict      = info[5]
 
-        base_learner_infos: Sequence[Optional[dict]] = []
-
         if self._type == "importance":
             # This is what is in the original paper. It has the following characteristics:
             #   > It is able to provide feedback to every base learner on every iteration
@@ -121,7 +128,7 @@ class CorralLearner(Learner):
             # The reward, R, supplied to the base learners satisifies E[R|context,A] = E[reward|context,A]
             for learner, A, P, base_info in zip(self._base_learners, base_actions, base_probs, base_infos):
                 R = reward * int(A==action)/probability
-                base_learner_infos.append(learner.learn(context, A, R, P, base_info))
+                learner.learn(context, A, R, P, base_info)
 
         if self._type == "off-policy":
             # An alternative variation to the paper is provided below. It has the following characterisitcs: 
@@ -129,7 +136,7 @@ class CorralLearner(Learner):
             #   > It uses a MVUB reward estimator (aka, the unmodified, observed reward)
             #   > It is "off-policy" (i.e., base learners receive action feedback distributed differently from their predicts).
             for learner, base_info in zip(self._base_learners, base_infos):
-                base_learner_infos.append(learner.learn(context, action, reward, probability, base_info))
+                learner.learn(context, action, reward, probability, base_info)
 
         if self._type == "rejection":
             # An alternative variation to the paper is provided below. It has the following characterisitcs: 
@@ -143,7 +150,7 @@ class CorralLearner(Learner):
                 
                 M = max([f(A)/g(A) for A in actions if g(A) > 0])
                 if p <= f(action)/(M*g(action)):
-                    base_learner_infos.append(learner.learn(context, action, reward, f(action), base_info))
+                    learner.learn(context, action, reward, f(action), base_info)
 
         # Instant loss is an unbiased estimate of E[loss|learner] for this iteration.
         # Our estimate differs from the orginal Corral paper because we have access to the
@@ -167,9 +174,7 @@ class CorralLearner(Learner):
         base_pbar_data    = { f"pbar_{i}"   : self._p_bars[i]             for i in range(len(self._base_learners)) }
         predict_data      = { "predict"     : probability, **base_predict_data, **base_pbar_data }
 
-        base_info = { k:v for b in filter(None,base_learner_infos) for k,v in b.items() }
-
-        return { k:round(v,4) for k,v in {**predict_data, **base_predict_data, **base_pbar_data, **base_info}.items() }
+        LearnerContext.logger.write({**predict_data, **base_predict_data, **base_pbar_data})
 
     @staticmethod
     def _log_barrier_omd(ps, losses, etas) -> Sequence[float]:
@@ -230,4 +235,4 @@ class CorralLearner(Learner):
 
         assert round(sum(new_ps),precision) == 1, "An invalid update was made by the log barrier in Corral"
 
-        return new_ps
\ No newline at end of file
+        return new_ps
