@@ -11,8 +11,6 @@ from itertools import islice
 from collections import OrderedDict
 from typing import Iterable, Any, Sequence, Union, Tuple, List, Dict, Callable, Optional
 
-from requests import Response
-
 from coba.random import CobaRandom
 from coba.encodings import Encoder, OneHotEncoder, NumericEncoder, StringEncoder, CobaJsonEncoder, CobaJsonDecoder
 from coba.exceptions import CobaException
@@ -27,26 +25,9 @@ _T_SparseData = Iterable[_T_SparseRow]
 _T_Row        = Union[_T_DenseRow,  _T_SparseRow ]
 _T_Data       = Union[_T_DenseData, _T_SparseData]
 
-class Cartesian(Filter[Union[Any,Iterable[Any]], Iterable[Any]]):
-
-    def __init__(self, filter: Union[Filter,Sequence[Filter]]):
-
-        self._filters = filter if isinstance(filter, collections.abc.Sequence) else [filter]
-
-    def filter(self, item: Union[Any,Iterable[Any]]) -> Iterable[Any]:
-
-        items = item if isinstance(item, collections.abc.Iterable) else [item]
-
-        for item in items:
-            for filter in self._filters:
-                yield filter.filter(item)
-
 class Identity(Filter[Any, Any]):
     def filter(self, item:Any) -> Any:
         return item
-    
-    def __repr__(self) -> str:
-        return "{ Identity }"
 
 class Shuffle(Filter[Iterable[Any], Iterable[Any]]):
 
@@ -57,35 +38,45 @@ class Shuffle(Filter[Iterable[Any], Iterable[Any]]):
 
         self._seed = seed
 
-    @property
-    def params(self) -> Dict[str, Any]:
-        return { "shuffle": self._seed }
-
     def filter(self, items: Iterable[Any]) -> Iterable[Any]: 
         return CobaRandom(self._seed).shuffle(list(items))
 
-    def __repr__(self) -> str:
-        return str(self.params)
-
 class Take(Filter[Iterable[Any], Iterable[Any]]):
     """Take a given number of items from an iterable."""
 
-    def __init__(self, count:Optional[int], seed: int = None, keep_first:bool = False) -> None:
+    def __init__(self, count:Optional[int]) -> None:
         """Instantiate a Take filter.
 
+        Args:
+            count: The number of items we wish to take from the given iterable.
+        """
+
+        if count is not None and (not isinstance(count,int) or count < 0):
+            raise ValueError(f"Invalid parameter for count: {count}. An optional integer value >= 0 was expected.")
+        
+        self._count = count
+
+    def filter(self, items: Iterable[Any]) -> Iterable[Any]:
+        items =  list(islice(items,self._count))
+        return items if len(items) == self._count else []
+
+class Reservoir(Filter[Iterable[Any], Iterable[Any]]):
+    def __init__(self, count:Optional[int], seed: int = 1, keep_first:bool = False) -> None:
+        """Instantiate a Resevoir filter.
+
         Args:
             count     : The number of items we wish to take from the given iterable.
-            keep_first: Indicates if the first row should be kept with take applied to the rest. Useful for files with headers.
             seed      : An optional random seed to determine which random count items to take.
+            keep_first: Indicate whether the first row should be kept as is (useful for files with headers).
 
         Remarks:
-            We use Algorithm L as described by Kim-Hung Li. (1994) to ranomdly take count items.
+            We use Algorithm L as described by Kim-Hung Li. (1994) to take a random count of items.
 
         References:
             Kim-Hung Li. 1994. Reservoir-sampling algorithms of time complexity O(n(1 + log(N/n))). 
             ACM Trans. Math. Softw. 20, 4 (Dec. 1994), 481–493. DOI:https://doi.org/10.1145/198429.198435
         """
-
+        
         if count is not None and (not isinstance(count,int) or count < 0):
             raise ValueError(f"Invalid parameter for Take: {count}. An optional integer value >= 0 was expected.")
 
@@ -93,62 +84,31 @@ class Take(Filter[Iterable[Any], Iterable[Any]]):
         self._seed       = seed
         self._keep_first = keep_first
 
-    @property
-    def params(self) -> Dict[str, Any]:
-
-        if self._seed is not None:
-            return { "take": self._count, "take_seed": self._seed }
-        else: 
-            return { "take": self._count }
-
     def filter(self, items: Iterable[Any]) -> Iterable[Any]:
 
-        if self._count is None: 
-            return items
-        else:
+        items    = iter(items)
+        first    = [next(items)] if self._keep_first else []
+        resevoir = list(islice(items,self._count))
 
-            items    = iter(items)
-            first    = [next(items)] if self._keep_first else []
-            resevoir = list(islice(items,self._count))
+        this_count = len(resevoir) if self._count is None else self._count
 
-            if self._seed is not None:
-                rng = CobaRandom(self._seed)
-                W = 1
+        if this_count == 0:
+            return []
 
-                try:
-                    while True:
-                        [r1,r2,r3] = rng.randoms(3)
-                        W = W * math.exp(math.log(r1)/self._count)
-                        S = math.floor(math.log(r2)/math.log(1-W))
-                        resevoir[int(r3*self._count-.001)] = next(itertools.islice(items,S,S+1))
-                except StopIteration:
-                    pass
+        if self._seed is not None:
+            rng = CobaRandom(self._seed)
+            W = 1
 
-            return itertools.chain( first, resevoir if len(resevoir) == self._count else [])
+            try:
+                while True:
+                    [r1,r2,r3] = rng.randoms(3)
+                    W = W * math.exp(math.log(r1)/this_count)
+                    S = math.floor(math.log(r2)/math.log(1-W))
+                    resevoir[int(r3*this_count-.001)] = next(itertools.islice(items,S,S+1))
+            except StopIteration:
+                pass
 
-    def __repr__(self) -> str:
-        return str(self.params)
-
-class StringJoin(Filter[Iterable[str], str]):
-
-    def __init__(self, separator:str = '') -> None:
-        self._separator = separator
-
-    def filter(self, item: Iterable[str]) -> str:
-        return self._separator.join(item)
-
-class ResponseToLines(Filter[Response, Iterable[str]]):
-    def filter(self, item: Response) -> Iterable[str]:
-
-        if item.status_code != 200:
-
-            message = (
-                f"The response from {item.url} reported an error. "
-                "The status and reason were {item.status_code}-{item.reason}.")
-
-            raise Exception(message) from None
-
-        return item.content.decode('utf-8').split('\n')
+        return itertools.chain( first, resevoir if len(resevoir) == this_count else [])
 
 class JsonEncode(Filter[Any, str]):
  
@@ -158,7 +118,7 @@ class JsonEncode(Filter[Any, str]):
         #WARNING: improves the performance of this method by a few percentage points. 
 
         #JsonEncoder writes floats with .0 regardless of if they are integers so we convert them to int to save space
-        #JsonEncoder also writes floats out 16 digits so we round down to 5 here to reduce file size
+        #JsonEncoder also writes floats out 16 digits so we truncate them to 5 digits here to reduce file size
 
         if isinstance(obj,tuple):
             obj = list(obj)
@@ -372,7 +332,7 @@ class Encode(Filter[_T_Data, _T_Data]):
 
             if header is None:
                 yield header
-                header_synced_encoders = dict(self._encoders)
+                header_synced_encoders = self._encoders
 
             elif isinstance(header, dict):
                 yield header
@@ -519,7 +479,7 @@ class Default(Filter[_T_Data, _T_Data]):
 
             if header is None:
                 yield header
-                header_synced_defaults = dict(self._defaults)
+                header_synced_defaults = self._defaults
 
             elif isinstance(header, dict):
                 yield header
