@@ -6,11 +6,9 @@ and for the core functionality to specify classes creation recipes in config fil
 
 from itertools import repeat
 from importlib import reload
-
-from importlib_metadata import entry_points #type: ignore 
-
 from typing import Dict, Any, Callable
 
+from coba.backport.metadata import entry_points 
 from coba.exceptions import CobaException
 
 def coba_registry_class(name:str) -> Callable[[type],type]:
@@ -50,7 +48,7 @@ class CobaRegistry:
         kwargs = None
         method = "singular"
 
-        if not cls._is_valid_recipe(recipe):
+        if not cls.is_valid_recipe(recipe):
             raise CobaException(f"Invalid recipe {str(recipe)}")
 
         if isinstance(recipe, str):
@@ -84,13 +82,13 @@ class CobaRegistry:
         
         if not cls._endpoints_loaded:
             cls._endpoints_loaded = True
-            for ep in entry_points().select(group='coba.register'):
+            for ep in entry_points(group='coba.register'):
                 reload(ep.load()) #we use reload in case the registry has been cleared at some point
 
         return cls._registry
 
     @classmethod
-    def _is_valid_recipe(cls, recipe:Any) -> bool:
+    def is_valid_recipe(cls, recipe:Any) -> bool:
 
         if isinstance(recipe, str):
             return True
@@ -117,7 +115,7 @@ class CobaRegistry:
     @classmethod
     def is_known_recipe(cls, recipe:Any) -> bool:
 
-        if not cls._is_valid_recipe(recipe):
+        if not cls.is_valid_recipe(recipe):
             return False
 
         name = None
@@ -136,7 +134,7 @@ class CobaRegistry:
 
     @classmethod
     def _construct_or_return(cls, item:Any):        
-        return cls.construct(item) if cls._is_valid_recipe(item) and cls.is_known_recipe(item) else item
+        return cls.construct(item) if cls.is_valid_recipe(item) and cls.is_known_recipe(item) else item
 
     @classmethod
     def _construct_single(cls, recipe, name, args, kwargs) -> Any:
@@ -151,7 +149,7 @@ class CobaRegistry:
 
             if args is not None and kwargs is not None:
                 return cls.retrieve(name)(*args, **kwargs)
-            
+
             elif args is not None and kwargs is None:
                 try:
                     return cls.retrieve(name)(*args)
@@ -167,4 +165,4 @@ class CobaRegistry:
             raise CobaException(f"Unknown recipe {str(recipe)}")
 
         except Exception as e:
-            raise CobaException(f"Unable to create recipe {str(recipe)}")
\ No newline at end of file
+            raise CobaException(f"Unable to create recipe {str(recipe)}")
